//
//  RAOpenQuickerPlugin.m
//  OpenQuicker
//
//  Created by Quentin Carnicelli on 11/20/04.
//  Copyright 2004 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "RAOpenQuickerPlugin.h"

#import "RAOpenQuickerPanel.h"
#import "RAOpenQuickerMultiplePanel.h"
#import "RASourceFileFinder.h"

@implementation RAOpenQuickerPlugin

static id _sharedInstance = nil;

+ (void)pluginDidLoad: (NSBundle*)bundle
{
	if( _sharedInstance == nil )
	{
		_sharedInstance = [[self alloc] init];
	}
}

- (id)init
{
	self = [super init];
	
	if( self )
	{
		[self _insertMenu];
	}
	
	return self;
}

- (void)dealloc
{
	[mFileFinder release];
	[super dealloc];
}

- (void)_insertMenu
{
	NSMenu* menu;
	NSMenuItem* menuItem;
	int i;

	menu = [[NSApplication sharedApplication] mainMenu];
	menu = [[menu itemWithTitle: @"File"] submenu];

	for( i = 0; i < [menu numberOfItems]; i++ )
	{
		menuItem = (NSMenuItem*)[menu itemAtIndex: i];
		if( [menuItem action] == @selector( openQuicklyAction: ) )
			break;
		else
			menuItem = nil;
	}
	
	if( menuItem )
	{
		[menuItem setTarget: self];
		[menuItem setAction: @selector( openQuicklyAction: )];
		[menuItem setTag: '0pNQ'];
	}
}

- (NSString*)_frontSelectedText
{
	NSWindow* window;
	id textEditor;
	NSRange selRange;
	NSString* selection = nil;
	
	NS_DURING

	window = [[NSApplication sharedApplication] keyWindow];
	if( window )
	{
		textEditor = [window firstResponder];
		if( [textEditor isKindOfClass: [NSText class]] )
		{
			selRange = [textEditor selectedRange];
			if( selRange.location != NSNotFound && selRange.length > 0 )
			{
				selection = [[textEditor string] substringWithRange: selRange];
			}
		}
	}

	NS_HANDLER
		NSLog( @"%@", localException );
		selection = nil;
	NS_ENDHANDLER

	return selection;
}

- (NSString*)_frontProjectPath
{
	NSWindow* window;
	id delegate;
	id document;
	id project;
	NSString* path = nil;
	
	NS_DURING
		
	window = [[NSApplication sharedApplication] mainWindow];
	delegate = [window delegate];
	
	if( [delegate respondsToSelector: @selector( document )] )
	{
		document = [delegate performSelector: @selector( document )];

		if( [document respondsToSelector: @selector( project )] )
		{
			project = [document performSelector: @selector( project )];
			
			if( [project respondsToSelector: @selector( path )] )
			{
				path = [project path];
			}
		}
	}

	NS_HANDLER
		NSLog( @"%@", localException );
		path = nil;
	NS_ENDHANDLER

	if( path && ![path isKindOfClass: [NSString class]] )
		path = nil;

	return path;
}

#pragma mark -

- (RASourceFileFinder*)_fileFinder
{
	BOOL needsRefresh;
	NSString* projectPath;
	
	projectPath = [[self _frontProjectPath] stringByDeletingLastPathComponent];
	
	needsRefresh =  (mFileFinder == nil) ||
					(([NSDate timeIntervalSinceReferenceDate] - mFileFinderAge) > 300) ||
					(projectPath && [[mFileFinder searchPaths] containsObject: projectPath] == NO);

	if( needsRefresh )
	{
		[mFileFinder release];
		mFileFinder = nil;
	}

	if( mFileFinder == nil )
	{
		mFileFinder = [[RASourceFileFinder alloc] init];

		if( projectPath )
			[mFileFinder addSearchPaths: [NSArray arrayWithObject: projectPath] toDepth: 10];
		
		[mFileFinder addSearchPaths: [[NSUserDefaults standardUserDefaults] objectForKey: @"OpenQuicklyPaths"] toDepth: 1];
		
		mFileFinderAge = [NSDate timeIntervalSinceReferenceDate];
	}

	return mFileFinder;
}

- (void)_openFiles: (NSArray*)files
{
	NSApplication* app = [NSApplication sharedApplication];
	NSEnumerator* fileEnum = [files objectEnumerator];
	NSString* path;
	NSWindow* confuseItIntoWorking;
	
	if( [[NSApplication sharedApplication] keyWindow] != nil )
	{
		confuseItIntoWorking = [[NSWindow alloc] initWithContentRect: NSMakeRect(0,0,1,1) styleMask: NSTitledWindowMask+NSClosableWindowMask backing: NSBackingStoreBuffered defer: NO];
		[confuseItIntoWorking setReleasedWhenClosed: YES];
		[confuseItIntoWorking setAlphaValue: 0.0f];
		[confuseItIntoWorking makeKeyAndOrderFront: nil];
	}
	else
		confuseItIntoWorking = nil;

	while( (path = [fileEnum nextObject]) != nil )
	{
		[[app delegate] application: app openFile: path];

		[confuseItIntoWorking makeKeyAndOrderFront: nil];
	}
	
	[confuseItIntoWorking close];
}

- (IBAction)openQuicklyAction: (id)sender
{
	RASourceFileFinder* finder = [self _fileFinder];
	NSString* searchString = nil;
	NSArray* found = nil;
	
	//Look for selection	
	searchString = [self _frontSelectedText];

	if( [[NSUserDefaults standardUserDefaults] boolForKey: @"PBXSkipOpenQuicklyPanelIfPossible"] )
	{
		if( [searchString length] )
			found = [finder find: searchString];
	}

	//Not find anything? ask for better input
	if( !found )
	{
		searchString = [[RAOpenQuickerPanel sharedPanel] runModalWithText: searchString fileFinder: finder];
		if( [searchString length] )
			found = [finder find: searchString];
		else
			found = nil;
	}

	//Got anything?
	if( [found count] )
	{
		//Ask if we got multiple hits
		if( [found count] > 1 )
			found = [[RAOpenQuickerMultiplePanel sharedPanel] runModalForFiles: found];
	}

	//Open 'em!
	if( [found count] )
		[self _openFiles: found];
}

@end

