//
//  RASourceFileFinder.m
//  OpenQuicker
//
//  Created by Quentin Carnicelli on 11/20/04.
//  Copyright 2004 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "RASourceFileFinder.h"

typedef unsigned short RAFileDirectoryID;
typedef unsigned short RAFileProjectID;

enum{ RAFileProjectIDUnknown = 0xFFFF };

@interface _RASourceFile : NSObject
{
	NSString*			mName;
	NSString*			mExtension;
	RAFileDirectoryID	mDirectoryID;
	RAFileProjectID		mProjectID;
}

- (id)initWithBaseName: (NSString*)name extension: (NSString*)ext directoryID: (RAFileDirectoryID)dirID projectID: (RAFileProjectID)prjID;

- (NSString*)baseName;
- (NSString*)extension;
- (NSString*)fullName;
- (RAFileDirectoryID)directoryID;
- (RAFileProjectID)projectID;

@end

@implementation _RASourceFile

- (id)initWithBaseName: (NSString*)name extension: (NSString*)ext directoryID: (RAFileDirectoryID)dirID projectID: (RAFileProjectID)prjID
{
	self = [super init];
	if( self )
	{
		mName = [name retain];
		mExtension = [ext retain];
		mDirectoryID = dirID;
		mProjectID = prjID;
	}
	
	return self;
}

- (void)dealloc
{
	[mName release];
	[super dealloc];
}

- (NSString*)baseName				{ return mName; }
- (NSString*)extension				{ return mExtension; }
- (NSString*)fullName				{ return [mExtension length]? [mName stringByAppendingPathExtension: mExtension] : mName; }
- (RAFileDirectoryID)directoryID	{ return mDirectoryID; }
- (RAFileProjectID)projectID		{ return mProjectID; }

@end

#pragma mark -

@implementation RASourceFileFinder

- (id)init
{
	self = [super init];
	
	if( self )
	{
		mSearchPaths	= [[NSMutableArray alloc] initWithCapacity: 64];
		mDirectoryIndex	= [[NSMutableArray alloc] initWithCapacity: 128];
		mProjectIndex	= [[NSMutableArray alloc] initWithCapacity: 64];
		mFileEntries	= [[NSMutableArray alloc] initWithCapacity: 2560];
	}
	
	return self;
}

- (void)dealloc
{
	[mSearchPaths release];
	[mFileEntries release];
	[mProjectIndex release];
	[mDirectoryIndex release];

	[super dealloc];
}

- (NSString*)_projectNameForDirectory: (NSString*)directory
{
	NSArray* parts;
	NSString* part;
	int i;
	
	parts = [directory pathComponents];
	for( i = ([parts count]-1); i > 0; i-- )
	{
		part = [parts objectAtIndex: i];
		if( [[part pathExtension] isEqualToString: @"framework"] )
		{
			return [part stringByDeletingPathExtension];
		}
	}
	
	return nil;
}

- (void)_loadFromDirectory: (NSString*)directory currentDepth: (int)depth maxDepth: (int)maxDepth
{
	NSFileManager* fm = [NSFileManager defaultManager];
	RAFileDirectoryID directoryID;
	NSString* projectName;
	RAFileProjectID projectID;
	int i, count;
	BOOL isDir;
	NSArray* fileNames;
	NSString* filePath;
	NSString* fileName;
	NSString* fileExt;
	_RASourceFile* entry;

	//Update the directory index
	[mDirectoryIndex addObject: directory];
	directoryID = [mDirectoryIndex count]-1;

	//Update the project index
	projectName = [self _projectNameForDirectory: directory];
	if( !projectName )
	{
		projectID = RAFileProjectIDUnknown;
	}
	else
	{
		[mProjectIndex addObject: projectName];
		projectID = [mProjectIndex count]-1;
	}

	//Get all the sub-files
	fileNames = [fm directoryContentsAtPath: directory];
	count = [fileNames count];
	
	for( i = 0; i < count; i++ )
	{
		fileName = [fileNames objectAtIndex: i];

		//Skip invisible
		if( [fileName hasPrefix: @"."] || [fileName isEqualToString: @"build"] )
			continue;

		fileExt = [fileName pathExtension];
		if( ![fileExt length] )
			fileExt = @"";
		else if( [fileExt isEqualToString: @"o"] || //Probably lots of other stuff we want to ignore too that I can't remember
				 [fileExt isEqualToString: @"p"] ||
				 [fileExt isEqualToString: @"xcode"] ||
				 [fileExt isEqualToString: @"nib"] )
			continue;
		
		//Directories
		filePath = [directory stringByAppendingPathComponent: fileName];
		if( [fm fileExistsAtPath: filePath isDirectory: &isDir] == NO )
			continue;
		
		if( isDir )
		{
			if( depth < maxDepth )
			{
				[self _loadFromDirectory: filePath currentDepth: depth+1 maxDepth: maxDepth];
			}
		
			continue;
		}
		
		//Update the file index
		fileName = [fileName stringByDeletingPathExtension];
		if( ![fileName length] )
			fileName = @"";
			
		entry = [[_RASourceFile alloc] initWithBaseName: fileName extension: fileExt directoryID: directoryID projectID: projectID];
		[mFileEntries addObject: entry];
		[entry release];
	}
}


- (void)addSearchPaths: (NSArray*)searchPaths toDepth: (int)maxDepth
{
	NSAutoreleasePool* pool	= [[NSAutoreleasePool alloc] init];
	int i, count			= [searchPaths count];
	BOOL isDir;
	NSString* directory;
	
	[mSearchPaths addObjectsFromArray: searchPaths];
	
	for( i = 0; i < count; i++ )
	{
		directory = [searchPaths objectAtIndex: i];
		directory = [directory stringByExpandingTildeInPath];
		
		if( [[NSFileManager defaultManager] fileExistsAtPath: directory isDirectory: &isDir] && isDir )
			[self _loadFromDirectory: directory currentDepth: 1 maxDepth: maxDepth];
	}
	
	[pool release];
}

- (NSArray*)searchPaths
{
	return mSearchPaths;
}

- (NSArray*)_findDirectly: (NSString*)text
{
	NSString* path = [text stringByStandardizingPath];
	BOOL isDir;
	
	if( [[NSFileManager defaultManager] fileExistsAtPath: path isDirectory: &isDir] && !isDir )
		return [NSArray arrayWithObject: path];
	
	return nil;
}

- (NSArray*)_findInIndex: (NSString*)text
{
	int i, count			= [mFileEntries count];
	_RASourceFile* file		= nil;
	NSMutableArray*			foundFiles = [NSMutableArray arrayWithCapacity: 1];

	NSArray* parts;
	NSString* projectName;
	RAFileProjectID projectID;
	NSString* baseName;
	NSString* extension;
	NSString* fullPath;
	
		
	//Analyze the search string
	parts = [text pathComponents];
	
	//Figure out the project id
	if( [parts count] > 1 )
	{
		int index;

		projectID = RAFileProjectIDUnknown;

		projectName = [parts objectAtIndex: [parts count]-2];
		for( index = 0; index < [mProjectIndex count]; index++ )
		{
			if( [[mProjectIndex objectAtIndex: index] caseInsensitiveCompare: projectName] == NSOrderedSame )
			{
				projectID = index;
				break;
			}
		}

	}
	else
	{
		projectName = @"";
		projectID = RAFileProjectIDUnknown;
	}
	
	//And the base name and extension
	baseName = [parts lastObject];
	extension = [baseName pathExtension];
	
	baseName = [baseName stringByDeletingPathExtension];
	if( ![extension length] )
		extension = nil;
			
	//Search
	for( i = 0; i < count; i++ )
	{
		file = [mFileEntries objectAtIndex: i];
		
		if( projectID == RAFileProjectIDUnknown || projectID == [file projectID] )
		{
			if( extension == nil || ([extension caseInsensitiveCompare: [file extension]] == NSOrderedSame) )
			{
				if( [baseName caseInsensitiveCompare: [file baseName]] == NSOrderedSame )
				{
					fullPath = [mDirectoryIndex objectAtIndex: [file directoryID]];
					fullPath = [fullPath stringByAppendingPathComponent: [file fullName]];
					[foundFiles addObject: fullPath];
				}
			}
		}
	}

	//Done
	if( ![foundFiles count] )
		foundFiles = nil;

	return foundFiles; 
}

- (NSArray*)find: (NSString*)text
{
	NSMutableArray* all = [NSMutableArray arrayWithCapacity: 1];
	NSArray* found;

	found = [self _findDirectly: text];
	if( found && [found count] )
		[all addObjectsFromArray: found];
	
	found = [self _findInIndex: text];
	if( found && [found count] )
		[all addObjectsFromArray: found];
		
	if( ![all count] )
		all = nil;
	return all;
}

@end

#pragma mark -

@implementation RASourceFileFinder (UIAdditions)

- (NSArray*)wordCompletionsForFragment: (NSString*)searchFragment limit: (int)matchLimit
{
	int i, count;
	NSString* word;
	NSMutableSet* matches = [NSMutableSet setWithCapacity: 8];

	for( i = 0, count = [mFileEntries count]; i < count; i++ )
	{
		word = [[mFileEntries objectAtIndex: i] fullName];

		if( [word rangeOfString:searchFragment options:( NSCaseInsensitiveSearch | NSAnchoredSearch )].location == 0 )
			[matches addObject: word];
	}

	
	for( i = 0, count = [mProjectIndex count]; i < count; i++ )
	{
		word = [mProjectIndex objectAtIndex: i];

		if( [word rangeOfString:searchFragment options:( NSCaseInsensitiveSearch | NSAnchoredSearch )].location == 0 )
			[matches addObject: [word stringByAppendingString: @"/"]];
	}
	
	return [matches allObjects];
}

@end
